# -*- coding: utf-8 -*-

"""Top-level package for url2tree."""

__author__ = """Taeoh Kim"""
__email__ = 'kimtaeoh95@gmail.com'
__version__ = '0.1.0'

from url2tree.url2tree import UrlTree, UrlTreeNode
